@@ -77,13 +77,19 @@ struct VirtualReaderOptionsStruct {
     int cert_count;
 };
 
+enum {
+    USE_HW_NO,
+    USE_HW_YES,
+    USE_HW_REMOVABLE,
+};
+
 struct VCardEmulOptionsStruct {
     void *nss_db;
     VirtualReaderOptions *vreader;
     int vreader_count;
     VCardEmulType hw_card_type;
     const char *hw_type_params;
-    PRBool use_hw;
+    int use_hw;
 };
 
 static int nss_emul_init;
@@ -213,6 +219,28 @@ vcard_emul_map_error(int error)
     return VCARD7816_STATUS_EXC_ERROR_CHANGE;
 }
 
+/* get RSA bits */
+int
+vcard_emul_rsa_bits(VCardKey *key)
+{
+    SECKEYPublicKey *pub_key;
+    int bits = -1;
+
+    if (key == NULL) {
+        /* couldn't get the key, indicate that we aren't logged in */
+        return -1;
+    }
+    pub_key = CERT_ExtractPublicKey(key->cert);
+    if (pub_key == NULL) {
+        /* couldn't get the key, indicate that we aren't logged in */
+        return -1;
+    }
+
+    bits = SECKEY_PublicKeyStrengthInBits(pub_key);
+    SECKEY_DestroyPublicKey(pub_key);
+    return bits;
+}
+
 /* RSA sign/decrypt with the key, signature happens 'in place' */
 vcard_7816_status_t
 vcard_emul_rsa_op(VCard *card, VCardKey *key,
@@ -246,7 +274,7 @@ vcard_emul_rsa_op(VCard *card, VCardKey *key,
     if ((unsigned)buffer_size != signature_len) {
         return  VCARD7816_STATUS_ERROR_DATA_INVALID;
     }
-    /* be able to handle larger keys if necessariy */
+    /* be able to handle larger keys if necessary */
     bp = &buf[0];
     if (sizeof(buf) < signature_len) {
         bp = g_malloc(signature_len);
@@ -270,7 +298,7 @@ vcard_emul_rsa_op(VCard *card, VCardKey *key,
         }
         /*
          * we've had a successful X509 operation, this failure must be
-         * somethine else
+         * something else
          */
         if (key->failedX509 == VCardEmulFalse) {
             ret = vcard_emul_map_error(PORT_GetError());
@@ -323,6 +351,12 @@ vcard_emul_rsa_op(VCard *card, VCardKey *key,
             key->failedX509 = VCardEmulTrue;
             goto cleanup;
         }
+    } else {
+        /* We can not do raw RSA operation, nor the data looks like PKCS#1.5
+         * bail out.
+         */
+        ret = VCARD7816_STATUS_ERROR_DATA_INVALID;
+        goto cleanup;
     }
     pad_len = buffer_size - signature_len;
     assert(pad_len < 4);
@@ -396,6 +430,10 @@ vcard_emul_login(VCard *card, unsigned char *pin, int pin_len)
         pin_string[i] = 0;
     }
 
+    /* If using an emulated card, make sure to log out of any already logged in
+     * session. */
+    vcard_emul_logout(card);
+
     rv = PK11_Authenticate(slot, PR_FALSE, pin_string);
     memset(pin_string, 0, pin_len);  /* don't let the pin hang around in memory
                                         to be snooped */
@@ -483,6 +521,7 @@ vreader_emul_delete(VReaderEmul *vreader_emul)
     if (vreader_emul == NULL) {
         return;
     }
+    vcard_free(vreader_emul->saved_vcard);
     if (vreader_emul->slot) {
         PK11_FreeSlot(vreader_emul->slot);
     }
@@ -662,6 +701,7 @@ vcard_emul_mirror_card(VReader *vreader)
         cert_count++;
         CERT_DestroyCertificate(cert); /* key obj still has a reference */
     }
+    PK11_DestroyGenericObjects(firstObj);
 
     /* now create the card */
     card = vcard_emul_make_card(vreader, certs, cert_len, keys, cert_count);
@@ -777,7 +817,7 @@ static const VCardEmulOptions default_options = {
     .vreader_count = 0,
     .hw_card_type = VCARD_EMUL_CAC,
     .hw_type_params = "",
-    .use_hw = PR_TRUE
+    .use_hw = USE_HW_YES,
 };
 
 
@@ -844,30 +884,6 @@ vcard_emul_force_card_insert(VReader *vreader)
     return VCARD_EMUL_OK;
 }
 
-
-static PRBool
-module_has_removable_hw_slots(SECMODModule *mod)
-{
-    int i;
-    PRBool ret = PR_FALSE;
-    SECMODListLock *moduleLock = SECMOD_GetDefaultModuleListLock();
-
-    if (!moduleLock) {
-        PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
-        return ret;
-    }
-    SECMOD_GetReadLock(moduleLock);
-    for (i = 0; i < mod->slotCount; i++) {
-        PK11SlotInfo *slot = mod->slots[i];
-        if (PK11_IsRemovable(slot) && PK11_IsHW(slot)) {
-            ret = PR_TRUE;
-            break;
-        }
-    }
-    SECMOD_ReleaseReadLock(moduleLock);
-    return ret;
-}
-
 /* Previously we returned FAIL if no readers found. This makes
  * no sense when using hardware, since there may be no readers connected
  * at the time vcard_emul_init is called, but they will be properly
@@ -1007,14 +1023,6 @@ vcard_emul_init(const VCardEmulOptions *options)
     /* make sure we have some PKCS #11 module loaded */
     module_lock = SECMOD_GetDefaultModuleListLock();
     module_list = SECMOD_GetDefaultModuleList();
-    SECMOD_GetReadLock(module_lock);
-    for (mlp = module_list; mlp; mlp = mlp->next) {
-        SECMODModule *module = mlp->module;
-        if (module_has_removable_hw_slots(module)) {
-            break;
-        }
-    }
-    SECMOD_ReleaseReadLock(module_lock);
 
     /* now examine all the slots, finding which should be readers */
     /* We should control this with options. For now we mirror out any
@@ -1035,7 +1043,8 @@ vcard_emul_init(const VCardEmulOptions *options)
             PK11SlotInfo *slot = module->slots[i];
 
             /* only map removable HW slots */
-            if (slot == NULL || !PK11_IsRemovable(slot) || !PK11_IsHW(slot)) {
+            if (slot == NULL || !PK11_IsRemovable(slot) ||
+                (options->use_hw == USE_HW_YES && !PK11_IsHW(slot))) {
                 continue;
             }
             if (strcmp("E-Gate 0 0", PK11_GetSlotName(slot)) == 0) {
@@ -1233,9 +1242,11 @@ vcard_emul_options(const char *args)
         } else if (strncmp(args, "use_hw=", 7) == 0) {
             args = strip(args+7);
             if (*args == '0' || *args == 'N' || *args == 'n' || *args == 'F') {
-                opts->use_hw = PR_FALSE;
+                opts->use_hw = USE_HW_NO;
+            } else if (strncmp(args, "removable", 9) == 0) {
+                opts->use_hw = USE_HW_REMOVABLE;
             } else {
-                opts->use_hw = PR_TRUE;
+                opts->use_hw = USE_HW_YES;
             }
             args = find_blank(args);
         /* hw_type= */
@@ -1276,12 +1287,12 @@ vcard_emul_options(const char *args)
             }
         } else if (strncmp(args, "nssemul", 7) == 0) {
             opts->hw_card_type = VCARD_EMUL_CAC;
-            opts->use_hw = PR_TRUE;
+            opts->use_hw = USE_HW_YES;
             args = find_blank(args + 7);
 #if defined(ENABLE_PCSC)
         } else if (strncmp(args, "passthru", 8) == 0) {
             opts->hw_card_type = VCARD_EMUL_PASSTHRU;
-            opts->use_hw = PR_TRUE;
+            opts->use_hw = USE_HW_YES;
             args = find_blank(args + 8);
 #endif
         } else {
@@ -1293,13 +1304,64 @@ vcard_emul_options(const char *args)
     return opts;
 }
 
+unsigned char *
+vcard_emul_read_object(VCard *card, const char *label,
+    unsigned int *ret_len)
+{
+    PK11SlotInfo *slot;
+    PK11GenericObject *obj, *firstObj, *myObj = NULL;
+    SECItem result;
+    SECStatus r;
+
+    slot = vcard_emul_card_get_slot(card);
+
+    firstObj = PK11_FindGenericObjects(slot, CKO_DATA);
+    g_debug("%s: Search for generic objects: got %p", __func__, firstObj);
+    for (obj = firstObj; obj; obj = PK11_GetNextGenericObject(obj)) {
+        int found = 0;
+        r = PK11_ReadRawAttribute(PK11_TypeGeneric, obj,
+            CKA_LABEL, &result);
+        if (r != SECSuccess) {
+            PK11_DestroyGenericObjects(firstObj);
+            return NULL;
+        }
+
+        if (strlen(label) == result.len
+            && memcmp(label, result.data, result.len) == 0)
+            found = 1;
+
+        free(result.data);
+        result.data = NULL;
+
+        if (found) {
+            PK11_UnlinkGenericObject(obj);
+            myObj = obj;
+            break;
+        }
+    }
+    PK11_DestroyGenericObjects(firstObj);
+
+    if (!myObj)
+        return NULL;
+
+    r = PK11_ReadRawAttribute(PK11_TypeGeneric, myObj,
+        CKA_VALUE, &result);
+    PK11_DestroyGenericObject(myObj);
+    if (r != SECSuccess)
+        return NULL;
+
+    *ret_len = result.len;
+    return result.data;
+
+}
+
 void
 vcard_emul_usage(void)
 {
    fprintf(stderr,
 "emul args: comma separated list of the following arguments\n"
 " db={nss_database}               (default sql:/etc/pki/nssdb)\n"
-" use_hw=[yes|no]                 (default yes)\n"
+" use_hw=[yes|no|removable]       (default yes)\n"
 " hw_type={card_type_to_emulate}  (default CAC)\n"
 " hw_param={param_for_card}       (default \"\")\n"
 " nssemul                         (alias for use_hw=yes, hw_type=CAC)\n"
@@ -1320,7 +1382,8 @@ vcard_emul_usage(void)
 "\n"
 "Unless use_hw is set to no, all tokens that look like removable hardware\n"
 "tokens will be presented to the guest using the emulator specified by\n"
-"hw_type, and parameters of hw_param.\n"
+"hw_type, and parameters of hw_param. If use_hw is set to 'removable', "
+"present any removable token.\n"
 "\n"
 "If more one or more soft= parameters are specified, these readers will be\n"
 "presented to the guest\n"
@@ -1331,3 +1394,4 @@ vcard_emul_usage(void)
 #endif
 );
 }
+/* vim: set ts=4 sw=4 tw=0 noet expandtab: */
